"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.withKeyHandler = withKeyHandler;

var _extends2 = _interopRequireDefault(require("@babel/runtime/helpers/extends"));

var _react = _interopRequireDefault(require("react"));

var _shortid = _interopRequireDefault(require("shortid"));

var _isHotkey = _interopRequireDefault(require("is-hotkey"));

/**
 * This HOC currently only works with single instance of a component.
 * That is caused by ID generation for an entire component, not for an instance of the component.
 * If you need to change that - take a look at the `decorator` function at the bottom of the file and
 * try to adapt the logic to work with multiple instances.
 */
var keyStack = {};
var listener = false;
var filter = ["TEXTAREA", "INPUT"];

var setupListener = function setupListener() {
  if (!listener && document.body) {
    document.body.addEventListener("keydown", function (e) {
      // We ignore all keyboard events coming from within `slateEditor` element and inputs.
      if (e.srcElement.dataset.slateEditor || filter.includes(e.srcElement.nodeName)) {
        return;
      }

      var matchedKey = Object.keys(keyStack).find(function (key) {
        return (0, _isHotkey.default)(key, e);
      });

      if (matchedKey && keyStack[matchedKey].length > 0) {
        var item = keyStack[matchedKey][0];
        item.handler(e);
        e.stopPropagation();
      }
    });
    listener = true;
  }
};

var _addKeyHandler = function addKeyHandler(id, key, handler) {
  setupListener();
  keyStack[key] = keyStack[key] || [];

  if (!keyStack[key].find(function (item) {
    return item.id === id;
  })) {
    keyStack[key].unshift({
      id: id,
      handler: handler
    });
  }
};

var _removeKeyHandler = function removeKeyHandler(id, key) {
  if (!keyStack[key]) {
    return;
  }

  var index = keyStack[key].findIndex(function (item) {
    return item.id === id;
  });

  if (index >= 0) {
    keyStack[key].splice(index, 1);
  }
};

function withKeyHandler() {
  return function decorator(Component) {
    // Generate a unique ID for this component
    var id = _shortid.default.generate();

    var keyProps = {
      addKeyHandler: function addKeyHandler(key, handler) {
        _addKeyHandler(id, key, handler);
      },
      removeKeyHandler: function removeKeyHandler(key) {
        _removeKeyHandler(id, key);
      }
    };
    return function (props) {
      return _react.default.createElement(Component, (0, _extends2.default)({}, props, keyProps));
    };
  };
}
//# sourceMappingURL=withKeyHandler.js.map