"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = void 0;

var _pure2 = _interopRequireDefault(require("recompose/pure"));

var _withHandlers2 = _interopRequireDefault(require("recompose/withHandlers"));

var _withProps2 = _interopRequireDefault(require("recompose/withProps"));

var _withState2 = _interopRequireDefault(require("recompose/withState"));

var _compose2 = _interopRequireDefault(require("recompose/compose"));

var _react = _interopRequireDefault(require("react"));

var _slugify = _interopRequireDefault(require("slugify"));

var _redux = require("webiny-app-cms/editor/redux");

var _Input = require("webiny-ui/Input");

var _actions = require("webiny-app-cms/editor/actions");

var _selectors = require("webiny-app-cms/editor/selectors");

var _Tooltip = require("webiny-ui/Tooltip");

var _Typography = require("webiny-ui/Typography");

var _Styled = require("./Styled");

var Title = (0, _pure2.default)(function (_ref) {
  var editTitle = _ref.editTitle,
      enableEdit = _ref.enableEdit,
      setTitle = _ref.setTitle,
      title = _ref.title,
      onKeyDown = _ref.onKeyDown,
      onBlur = _ref.onBlur,
      pageCategory = _ref.pageCategory,
      pageLocked = _ref.pageLocked,
      pageVersion = _ref.pageVersion;
  return editTitle ? _react.default.createElement(_Styled.TitleInputWrapper, null, _react.default.createElement(_Input.Input, {
    autoFocus: true,
    fullwidth: true,
    value: title,
    onChange: setTitle,
    onKeyDown: onKeyDown,
    onBlur: onBlur
  })) : _react.default.createElement(_Styled.TitleWrapper, null, _react.default.createElement(_Styled.PageMeta, null, _react.default.createElement(_Typography.Typography, {
    use: "overline"
  }, "".concat(pageCategory, " (status: ").concat(pageLocked ? "published" : "draft", ")"))), _react.default.createElement("div", {
    style: {
      width: "100%",
      display: "flex"
    }
  }, _react.default.createElement(_Tooltip.Tooltip, {
    className: _Styled.pageTitleWrapper,
    placement: "bottom",
    content: _react.default.createElement("span", null, "Rename")
  }, _react.default.createElement(_Styled.PageTitle, {
    onClick: enableEdit
  }, title)), _react.default.createElement(_Styled.PageVersion, null, "(v".concat(pageVersion, ")"))));
});

var getRevData = function getRevData(_ref2) {
  var title = _ref2.title,
      pageTitle = _ref2.pageTitle,
      pageCategoryUrl = _ref2.pageCategoryUrl;
  var newData = {
    title: title
  };

  if (pageTitle === "Untitled") {
    newData.url = pageCategoryUrl + (0, _slugify.default)(title, {
      replacement: "-",
      lower: true,
      remove: /[*#\?<>_\{\}\[\]+~.()'"!:;@]/g
    });
  }

  return newData;
};

var _default = (0, _compose2.default)((0, _redux.connect)(function (state) {
  var _getPage = (0, _selectors.getPage)(state),
      title = _getPage.title,
      version = _getPage.version,
      locked = _getPage.locked,
      category = _getPage.category;

  return {
    pageTitle: title,
    pageVersion: version,
    pageLocked: locked,
    pageCategory: category.name,
    pageCategoryUrl: category.url
  };
}, {
  updateRevision: _actions.updateRevision
}), (0, _withState2.default)("editTitle", "setEdit", false), (0, _withState2.default)("title", "setTitle", null), (0, _withProps2.default)(function (_ref3) {
  var title = _ref3.title,
      pageTitle = _ref3.pageTitle;
  return {
    title: title === null ? pageTitle : title
  };
}), (0, _withHandlers2.default)({
  enableEdit: function enableEdit(_ref4) {
    var setEdit = _ref4.setEdit;
    return function () {
      return setEdit(true);
    };
  },
  onBlur: function onBlur(_ref5) {
    var title = _ref5.title,
        pageTitle = _ref5.pageTitle,
        pageCategoryUrl = _ref5.pageCategoryUrl,
        setEdit = _ref5.setEdit,
        setTitle = _ref5.setTitle,
        updateRevision = _ref5.updateRevision;
    return function () {
      if (title === "") {
        title = "Untitled";
        setTitle(title);
      }

      setEdit(false);
      updateRevision(getRevData({
        title: title,
        pageTitle: pageTitle,
        pageCategoryUrl: pageCategoryUrl
      }));
    };
  },
  onKeyDown: function onKeyDown(_ref6) {
    var title = _ref6.title,
        setTitle = _ref6.setTitle,
        setEdit = _ref6.setEdit,
        pageTitle = _ref6.pageTitle,
        pageCategoryUrl = _ref6.pageCategoryUrl,
        updateRevision = _ref6.updateRevision;
    return function (e) {
      switch (e.key) {
        case "Escape":
          e.preventDefault();
          setEdit(false);
          setTitle(pageTitle);
          break;

        case "Enter":
          if (title === "") {
            title = "Untitled";
            setTitle(title);
          }

          e.preventDefault();
          setEdit(false);
          updateRevision(getRevData({
            title: title,
            pageTitle: pageTitle,
            pageCategoryUrl: pageCategoryUrl
          }));
          break;

        default:
          return;
      }
    };
  }
}))(Title);

exports.default = _default;
//# sourceMappingURL=Title.js.map