"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = void 0;

var _objectSpread2 = _interopRequireDefault(require("@babel/runtime/helpers/objectSpread"));

var _react = _interopRequireDefault(require("react"));

var _reactEmotion = _interopRequireDefault(require("react-emotion"));

var _dotPropImmutable = require("dot-prop-immutable");

var _redux = require("webiny-app-cms/editor/redux");

var _Column = _interopRequireDefault(require("./Column"));

var _utils = require("webiny-app-cms/editor/utils");

var _actions = require("webiny-app-cms/editor/actions");

var _selectors = require("webiny-app-cms/editor/selectors");

var _columnIcon = require("@svgr/webpack!webiny-app-cms/editor/assets/icons/column-icon.svg");

var _default = function _default() {
  var PreviewBox =
  /*#__PURE__*/
  (0, _reactEmotion.default)("div", {
    target: "epvrohk0",
    label: "PreviewBox"
  })({
    textAlign: "center",
    height: 50,
    svg: {
      height: 50,
      width: 50
    }
  });
  return {
    name: "cms-element-column",
    type: "cms-element",
    toolbar: {
      title: "Column",
      group: "cms-element-group-layout",
      preview: function preview() {
        return _react.default.createElement(PreviewBox, null, _react.default.createElement(_columnIcon.ReactComponent, null));
      }
    },
    settings: ["cms-element-settings-background", "cms-element-settings-animation", "", "cms-element-settings-border", "cms-element-settings-shadow", "", "cms-element-settings-padding", "cms-element-settings-margin", "cms-element-settings-horizontal-align", "cms-element-settings-vertical-align", "", "cms-element-settings-clone", "cms-element-settings-delete", ""],
    target: ["cms-element-row"],
    create: function create() {
      var options = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : {};
      return {
        type: "cms-element-column",
        data: (0, _objectSpread2.default)({}, options.data || {}, {
          settings: {
            margin: {
              desktop: {
                all: 0
              },
              mobile: {
                all: 0
              }
            },
            padding: {
              desktop: {
                all: 0
              },
              mobile: {
                all: 0
              }
            }
          }
        }),
        elements: options.elements || []
      };
    },
    render: function render(props) {
      return _react.default.createElement(_Column.default, props);
    },
    canDelete: function canDelete(_ref) {
      var element = _ref.element;
      var parent = (0, _selectors.getParentElementWithChildren)(_redux.redux.store.getState(), element.id);
      return parent.elements.length > 1;
    },
    onReceived: function onReceived(_ref2) {
      var source = _ref2.source,
          target = _ref2.target,
          _ref2$position = _ref2.position,
          position = _ref2$position === void 0 ? null : _ref2$position;
      var droppedOnCenter = position === null; // Dropped a column onto a center drop zone

      if (source.type === "cms-element-column" && droppedOnCenter) {
        return splitColumn(source, target);
      }

      handleDroppedElement(source, target, position);
    }
  };
};

exports.default = _default;

var handleDroppedElement = function handleDroppedElement(source, target, position) {
  var dispatchNew = false;
  var element;

  if (source.path) {
    // $FlowFixMe
    element = (0, _utils.cloneElement)(source);
  } else {
    dispatchNew = true;
    element = (0, _utils.createElement)(source.type, {}, target);
  }

  target = (0, _utils.addElementToParent)(element, target, position);

  _redux.redux.store.dispatch((0, _actions.updateElement)({
    element: target
  }));

  if (source.path) {
    _redux.redux.store.dispatch((0, _actions.deleteElement)({
      element: source
    }));
  }

  if (dispatchNew) {
    _redux.redux.store.dispatch((0, _actions.elementCreated)({
      element: element,
      source: source
    }));
  }
};

var splitColumn = function splitColumn(source, target) {
  var dispatchNew = false;
  var row = (0, _selectors.getParentElementWithChildren)(_redux.redux.store.getState(), target.id); // $FlowFixMe

  var targetIndex = row.elements.findIndex(function (el) {
    return el.id === target.id;
  }); // Split target column in half
  // $FlowFixMe

  row.elements[targetIndex].data.width /= 2; // Create a new column with half of the original target width

  var newColumn;

  if (source.path) {
    // $FlowFixMe
    newColumn = (0, _utils.cloneElement)(source);
  } else {
    dispatchNew = true;
    newColumn = (0, _utils.createColumn)();
  } // $FlowFixMe


  newColumn = (0, _dotPropImmutable.set)(newColumn, "data.width", row.elements[targetIndex].data.width);
  row = (0, _utils.addElementToParent)(newColumn, row, targetIndex + 1);

  _redux.redux.store.dispatch((0, _actions.updateElement)({
    element: row
  }));

  if (source.path) {
    _redux.redux.store.dispatch((0, _actions.deleteElement)({
      element: source
    }));
  }

  if (dispatchNew) {
    _redux.redux.store.dispatch((0, _actions.elementCreated)({
      element: newColumn,
      source: source
    }));
  }
};
//# sourceMappingURL=index.js.map