"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = void 0;

var _objectSpread2 = _interopRequireDefault(require("@babel/runtime/helpers/objectSpread"));

var _react = _interopRequireDefault(require("react"));

var _reactEmotion = _interopRequireDefault(require("react-emotion"));

var _dotPropImmutable = require("dot-prop-immutable");

var _redux = require("webiny-app-cms/editor/redux");

var _utils = require("webiny-app-cms/editor/utils");

var _actions = require("webiny-app-cms/editor/actions");

require("./actions");

var _Row = _interopRequireDefault(require("./Row"));

var _rowIcon = require("@svgr/webpack!webiny-app-cms/editor/assets/icons/row-icon.svg");

var _default = function _default() {
  var PreviewBox =
  /*#__PURE__*/
  (0, _reactEmotion.default)("div", {
    target: "e62c7gu0",
    label: "PreviewBox"
  })({
    textAlign: "center",
    height: 50,
    svg: {
      height: 50,
      width: 50
    }
  });
  return {
    name: "cms-element-row",
    type: "cms-element",
    toolbar: {
      title: "Row",
      group: "cms-element-group-layout",
      // Render element preview
      preview: function preview() {
        return _react.default.createElement(PreviewBox, null, _react.default.createElement(_rowIcon.ReactComponent, null));
      }
    },
    settings: ["cms-element-settings-background", "cms-element-settings-animation", "", "cms-element-settings-border", "cms-element-settings-shadow", "", "cms-element-settings-padding", "cms-element-settings-margin", "cms-element-settings-width", "", "cms-element-settings-clone", "cms-element-settings-delete", ""],
    // Target drop zones that will accept this type
    target: ["cms-element-block", "cms-element-column"],
    // This function is called when `createElement` is called for this plugin
    create: function create() {
      var options = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : {};
      var row = (0, _objectSpread2.default)({
        type: "cms-element-row",
        elements: [],
        data: {
          settings: {
            margin: {
              desktop: {
                all: 0
              },
              mobile: {
                all: 0
              }
            },
            padding: {
              desktop: {
                all: 0
              },
              mobile: {
                all: 0
              }
            }
          }
        }
      }, options); // A row MUST contain at least 1 column

      if (!row.elements.length) {
        row.elements.push((0, _utils.createColumn)({
          data: {
            width: 100
          }
        }));
      }

      return row;
    },
    // Render element in editor
    render: function render(props) {
      return _react.default.createElement(_Row.default, props);
    },
    // This callback is executed when another element is dropped on the drop zones with type "row"
    onReceived: function onReceived(_ref) {
      var source = _ref.source,
          target = _ref.target,
          _ref$position = _ref.position,
          position = _ref$position === void 0 ? null : _ref$position;
      var dispatchNew = false;
      var element;

      if (source.path) {
        // $FlowFixMe
        element = (0, _utils.cloneElement)(source);
      } else {
        dispatchNew = true;
        element = (0, _utils.createElement)(source.type, {}, target);
      }

      if (element.type !== "cms-element-column") {
        element = (0, _utils.createColumn)({
          elements: [element]
        });
      } // Add new child element


      var row = (0, _utils.addElementToParent)(element, target, position); // Recalculate column widths

      row = distributeColumnWidths(row); // Dispatch update action

      (0, _redux.dispatch)((0, _actions.updateElement)({
        element: row
      }));

      if (source.path) {
        (0, _redux.dispatch)((0, _actions.deleteElement)({
          element: source
        }));
      }

      if (dispatchNew) {
        (0, _redux.dispatch)((0, _actions.elementCreated)({
          element: element,
          source: source
        }));
      }
    },
    onChildDeleted: function onChildDeleted(_ref2) {
      var element = _ref2.element;
      (0, _redux.dispatch)((0, _actions.updateElement)({
        element: distributeColumnWidths(element)
      }));
    }
  };
};

exports.default = _default;

var distributeColumnWidths = function distributeColumnWidths(row) {
  var width = Math.round(100 / row.elements.length * 100) / 100;
  var columns = row.elements.map(function (el) {
    return (0, _dotPropImmutable.set)(el, "data.width", width);
  });
  return (0, _dotPropImmutable.set)(row, "elements", columns);
};
//# sourceMappingURL=index.js.map