"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.getElementProps = exports.getIsDragging = exports.isPluginActive = exports.getActivePluginParams = exports.getActivePlugins = exports.getPluginsByType = exports.getPlugins = exports.getActiveElementId = exports.getActiveElement = exports.getParentElementWithChildren = exports.getElement = exports.getElementWithChildren = exports.getContent = exports.getRevisions = exports.getPage = exports.getUi = void 0;

var _get2 = _interopRequireDefault(require("lodash/get"));

var _invariant = _interopRequireDefault(require("invariant"));

var _webinyPlugins = require("webiny-plugins");

var getPluginType = function getPluginType(name) {
  var plugin = (0, _webinyPlugins.getPlugin)(name);
  return plugin ? plugin.type : null;
};
/**
 * Get editor `ui` state
 */


var getUi = function getUi(state) {
  return state.ui || {};
};
/**
 * Get editor `page` state
 */


exports.getUi = getUi;

var getPage = function getPage(state) {
  return state.page || {};
};
/**
 * Get editor `revisions` state
 */


exports.getPage = getPage;

var getRevisions = function getRevisions(state) {
  return state.revisions || [];
};
/**
 * Get editor content.
 */


exports.getRevisions = getRevisions;

var getContent = function getContent(state) {
  var page = getPage(state);

  if (page.content && page.content.present) {
    return page.content.present;
  } else if (page.content) {
    return page.content;
  }

  var document = (0, _webinyPlugins.getPlugin)("cms-element-document");
  (0, _invariant.default)(document, "\"cms-element-document\" plugin must exist for CMS to work!");
  return document.create();
};
/**
 * Get element and all of its children recursively.
 * WARNING: use carefully as this makes render optimization really difficult when used in `connect`!
 */


exports.getContent = getContent;

var getElementWithChildren = function getElementWithChildren(state, id) {
  var element = getElement(state, id);
  var content = getContent(state);
  return (0, _get2.default)(content, element.path.replace(/\./g, ".elements.").slice(2));
};
/**
 * Get element by ID or path.
 * @param state
 * @param id ID or path of the element
 */


exports.getElementWithChildren = getElementWithChildren;

var getElement = function getElement(state, id) {
  if (state.elements.hasOwnProperty(id)) {
    return state.elements[id];
  } // Find by path
  // $FlowFixMe


  return Object.values(state.elements).find(function (el) {
    return el.path === id;
  });
};
/**
 * Get parent element and all of its children recursively
 * WARNING: use carefully as this makes render optimization really difficult when used in `connect`!
 * @param state
 * @param id
 * @returns {*}
 */


exports.getElement = getElement;

var getParentElementWithChildren = function getParentElementWithChildren(state, id) {
  var element = getElement(state, id);
  var content = getContent(state);
  var parentPaths = element.path.split(".").slice(0, -1);

  if (parentPaths.length === 1) {
    return content;
  }

  return (0, _get2.default)(content, parentPaths.join(".elements.").slice(2));
};

exports.getParentElementWithChildren = getParentElementWithChildren;

var getActiveElement = function getActiveElement(state) {
  return getElement(state, getActiveElementId(state));
};
/**
 * Get active element ID.
 */


exports.getActiveElement = getActiveElement;

var getActiveElementId = function getActiveElementId(state) {
  return getUi(state).activeElement;
};
/**
 * Get editor plugins (this mostly contains UI state).
 */


exports.getActiveElementId = getActiveElementId;

var getPlugins = function getPlugins(state) {
  return getUi(state).plugins || {};
};
/**
 * Get editor plugins of certain type (this mostly contains UI state).
 */


exports.getPlugins = getPlugins;

var getPluginsByType = function getPluginsByType(type) {
  return function (state) {
    return getPlugins(state)[type] || [];
  };
};
/**
 * Get an active plugin of the given type.
 */


exports.getPluginsByType = getPluginsByType;

var getActivePlugins = function getActivePlugins(type) {
  var pluginsByType = getPluginsByType(type);
  return function (state) {
    return pluginsByType(state) || [];
  };
};
/**
 * Get active plugin params
 */


exports.getActivePlugins = getActivePlugins;

var getActivePluginParams = function getActivePluginParams(name) {
  var type = getPluginType(name);

  if (typeof type !== "string") {
    return null;
  }

  var pluginsByType = getPluginsByType(type);
  return function (state) {
    var plugins = pluginsByType(state);

    if (plugins) {
      var plugin = plugins.find(function (pl) {
        return pl.name === name;
      });
      return plugin ? plugin.params : null;
    }

    return null;
  };
};

exports.getActivePluginParams = getActivePluginParams;

var isPluginActive = function isPluginActive(name) {
  var type = getPluginType(name);

  if (typeof type !== "string") {
    // eslint-disable-next-line
    return function (state) {
      return false;
    };
  }

  var pluginsByType = getPluginsByType(type);
  return function (state) {
    return Boolean(pluginsByType(state).find(function (pl) {
      return pl.name === name;
    }));
  };
};
/**
 * Get dragging state.
 */


exports.isPluginActive = isPluginActive;

var getIsDragging = function getIsDragging(state) {
  return getUi(state).dragging;
};
/**
 * Get props that need to be passed to an <Element>.
 */


exports.getIsDragging = getIsDragging;

var getElementProps = function getElementProps(state, _ref) {
  var id = _ref.id;

  var _getUi = getUi(state),
      activeElement = _getUi.activeElement,
      highlightElement = _getUi.highlightElement,
      resizing = _getUi.resizing,
      dragging = _getUi.dragging;

  var element = state.elements[id];
  var active = activeElement && activeElement === element.id;
  var highlight = active || highlightElement && highlightElement === id;
  return {
    active: active,
    highlight: highlight && !dragging && !resizing
  };
};

exports.getElementProps = getElementProps;
//# sourceMappingURL=index.js.map