"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.cloneElement = exports.createElement = exports.createColumn = exports.createRow = exports.createBlock = exports.addElementToParent = exports.updateChildPaths = void 0;

var _objectSpread2 = _interopRequireDefault(require("@babel/runtime/helpers/objectSpread"));

var _toConsumableArray2 = _interopRequireDefault(require("@babel/runtime/helpers/toConsumableArray"));

var _omit2 = _interopRequireDefault(require("lodash/omit"));

var _isPlainObject2 = _interopRequireDefault(require("lodash/isPlainObject"));

var _shortid = _interopRequireDefault(require("shortid"));

var _invariant = _interopRequireDefault(require("invariant"));

var _dotPropImmutable = require("dot-prop-immutable");

var _webinyPlugins = require("webiny-plugins");

var updateChildPaths = function updateChildPaths(element) {
  if (!element.id) {
    element.id = _shortid.default.generate();
  }

  if (!element.path) {
    element.path = "0";
  }

  if (Array.isArray(element.elements)) {
    // Process children only if "elements" is an array of objects.
    // We may get an array of strings when working with shallow element copies.
    if ((0, _isPlainObject2.default)(element.elements[0])) {
      element.elements.forEach(function (el, index) {
        if (!el.id) {
          el.id = _shortid.default.generate();
        }

        el.path = element.path + "." + index;

        if (el.elements.length) {
          updateChildPaths(el);
        }
      });
    }
  }
};

exports.updateChildPaths = updateChildPaths;

var addElementToParent = function addElementToParent(element, parent, position) {
  var newParent;

  if (position === null) {
    newParent = (0, _dotPropImmutable.set)(parent, "elements", [].concat((0, _toConsumableArray2.default)(parent.elements), [element]));
  } else {
    newParent = (0, _dotPropImmutable.set)(parent, "elements", [].concat((0, _toConsumableArray2.default)(parent.elements.slice(0, position)), [element], (0, _toConsumableArray2.default)(parent.elements.slice(position))));
  }

  updateChildPaths(newParent);
  return newParent;
};

exports.addElementToParent = addElementToParent;

var createBlock = function createBlock() {
  var options = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : {};
  var parent = arguments.length > 1 ? arguments[1] : undefined;
  return createElement("cms-element-block", options, parent);
};

exports.createBlock = createBlock;

var createRow = function createRow() {
  var options = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : {};
  var parent = arguments.length > 1 ? arguments[1] : undefined;
  return createElement("cms-element-row", options, parent);
};

exports.createRow = createRow;

var createColumn = function createColumn() {
  var options = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : {};
  var parent = arguments.length > 1 ? arguments[1] : undefined;
  return createElement("cms-element-column", options, parent);
};

exports.createColumn = createColumn;

var createElement = function createElement(type) {
  var options = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : {};
  var parent = arguments.length > 2 ? arguments[2] : undefined;
  var plugin = (0, _webinyPlugins.getPlugin)(type);
  (0, _invariant.default)(plugin, "Missing element plugin \"".concat(type, "\"!"));
  return (0, _objectSpread2.default)({
    id: _shortid.default.generate(),
    data: {},
    elements: [],
    path: ""
  }, plugin.create(options, parent));
};

exports.createElement = createElement;

var cloneElement = function cloneElement(element) {
  var clone = (0, _omit2.default)(element, ["id", "path"]);
  clone.elements = clone.elements.map(function (el) {
    return cloneElement(el);
  });
  return clone;
};

exports.cloneElement = cloneElement;
//# sourceMappingURL=utils.js.map